/** Autograde -- grading helper program for trace
  * By Andrew Lewis <foon@cs> Spring 2005 */

#include "imageio.h"
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdlib.h>
#include <direct.h>

#ifdef WIN32
	#define DIR_SEPARATOR "\\"
#else
	#define DIR_SEPARATOR "/"
#endif

#define DEFAULT_WIDTH		400
#define DEFAULT_IMAGE_CACHE	"cache"
#define MAX_WIDTH			512
#define MIN_WIDTH			200

using namespace std;

//calculate mean square deviation between images -- assumed to be same size
double deviation (unsigned char *img1, unsigned char *img2, int len) {
	double deviat = 0;
	for (int i = 0; i < len; i++) {
		double dev = (img1[i] - img2[i]);
		deviat += dev * dev;
	}
	
	deviat = deviat / len;
	return deviat;
}

//compute diff image and write to disk
void diffimg (unsigned char *img1, unsigned char *img2, int width, int height, const string &filename) {
	int len = 3*width*height;
	unsigned char *out = new unsigned char [len];
	for (int i = 0; i < len; i++) {
		out[i] = (img1[i] != img2[i]) ? 255 : 0; 
	}
	
	save(filename.c_str(), out, width, height, FILETYPE_PNG, 95);
}


int main(int argc, char **argv) {
	//Parse args
	if(argc < 3) {
		cout << "Usage: " << argv[0] << " <sample exe> <your exe> [opts]" << endl;
		return -1;
	}
	
	int width = DEFAULT_WIDTH;
	string sample_exe(argv[1]);
	string student_exe(argv[2]);
	string cache_dir = DEFAULT_IMAGE_CACHE;
	for(int i = 3; i < argc; i++) {
		switch(argv[i][1]) {
			case 'c':
				//image cache
				cache_dir = argv[++i];
				break;
			case 'w':
				//manual width control
				width = atoi(argv[++i]);
				if(width > MAX_WIDTH) width = MAX_WIDTH;
				if(width < MIN_WIDTH) width = MIN_WIDTH;
				break;
			case 'r':
				//random width
				width = (rand() % (MAX_WIDTH - MIN_WIDTH)) + MIN_WIDTH;
				break;
		}
	}
	
	//Make sure cache directory exists
	_mkdir(cache_dir.c_str());
	
	//Open settings file, run tests
	ifstream instream("autograde.rc"); //should really check for failure here...
	string scene_dir;
	getline(instream, scene_dir); //read the scene directory
	while(instream.good()){ //for each scene file in autograde.rc ...
		string scene;
		getline(instream, scene);
		cout << "* Testing " << scene << endl;
		string filename = cache_dir + DIR_SEPARATOR + scene.substr(scene.find_last_of(DIR_SEPARATOR)+1);
		//Run for depths 0 through 3
		for(int depth = 0; depth < 4; depth++) {
			//Build command
			cout << "** Depth = " << depth << endl;
			ostringstream args, sample_file, student_file, diff_file;
			args << " -r " << depth << " -w " << width << " -b ";	//depth .. width .. BSP accel
			sample_file << filename << "_sample_" << depth << ".png";
			student_file << filename << "_student_" << depth << ".png";
			diff_file << filename << "_diff_" << depth << ".png";
			
			string command = sample_exe + args.str() + scene_dir + DIR_SEPARATOR + scene + " " + sample_file.str();
			system(command.c_str()); //Run the sample
			command = student_exe + args.str() + scene_dir + DIR_SEPARATOR + scene + " " + student_file.str();
			system(command.c_str()); //Run the student's
			
			//now compute the diff images and deviation
			
			int w, h;	//To pass to readBMP
			
			//should check these for failure also
			unsigned char *sample_data = load(sample_file.str().c_str(), w, h);
			unsigned char *student_data = load(student_file.str().c_str(), w, h);
			double dev = deviation(sample_data, student_data, 3*w*h);
			std::cout << "Deviation: " << dev << std::endl;
			diffimg(sample_data, student_data, w, h, diff_file.str());
		}
	}

	return 0;
}